SET(CMAKE_SYSTEM_NAME QNX)
SET(CMAKE_SYSTEM_VERSION 6.6.0)
SET(ARM 1)
SET(CMAKE_SYSTEM_PROCESSOR ARM)

# we use qcc for compiling
set(CMAKE_COMPILER_IS_QCC 1)


set(EB_PLATFORM_OS_FAMILY POSIX)
set(EB_PLATFORM_OS QNX)
set(EB_PLATFORM_ARCH ARM)
set(CPU_ARM_SOC_SAVE NOT-DEFINED)

## The following to env variables must be defined, otherwise gcc wont work
SET(QNX_TARGET $ENV{QNX_TARGET} CACHE PATH "Path to QNX Target env")
SET(QNX_HOST $ENV{QNX_HOST} CACHE PATH "Path to QNX host tool chain")
# hence we reuse the variables, but the Cygwin paths must be converted into
# windows paths before that:

IF(QNX_HOST STREQUAL "")
    MESSAGE(SEND_ERROR "Error: No QNX build environment specified. Use the environment variable QNX_HOST for host environment!")
ELSE(QNX_HOST STREQUAL "")
    SET(QNX_HOST_CMAKE_PATH ${QNX_HOST})
ENDIF(QNX_HOST STREQUAL "")

IF(QNX_TARGET STREQUAL "")
    MESSAGE(SEND_ERROR "Error: No QNX build environment specified. Use the environment variables QNX_TARGET for target environment!")
ELSE(QNX_TARGET STREQUAL "")
    SET(QNX_TARGET_CMAKE_PATH ${QNX_TARGET})
ENDIF(QNX_TARGET STREQUAL "")

MESSAGE(STATUS "Compiling for QNX 6.6.0 ARM")



# Added here so the platform file can be used for other projects, too:
ADD_DEFINITIONS(
    -D_QNX_SOURCE  # Required for the gnu headers: http://gcc.gnu.org/ml/gcc/2004-04/msg00230.html
)

SET(CMAKE_FIND_ROOT_PATH ${QNX_HOST_CMAKE_PATH} ${QNX_TARGET_CMAKE_PATH})

SET(CPU_ARM_SOC ${CPU_ARM_SOC} CACHE STRING "Select ARM SoC")

SET(QCC_GCC_BACKEND_VERSION "4.7.3" CACHE STRING "Select GCC version used by QCC")
SET_PROPERTY(CACHE QCC_GCC_BACKEND_VERSION PROPERTY STRINGS 4.7.3)

SET(CPU_ARM "")

if (CPU_ARM_SOC STREQUAL "Cortex-A8")
    MESSAGE(STATUS "CPU_ARM_SOC: Cortex-A8")
    SET(CPU_ARM_ARCH "armv7-a")
    SET(CPU_QNX_PATH "armv7-le")
    SET(CPU_ARM_SPECIFIC "cortex-a8")
    SET(CPU_ARM_FLOAT_ABI "softfp")
    SET(CPU_ARM_FPU "neon")
    SET(QCC_GCC_BACKEND_TYPE "gcc_ntoarmv7le_cpp-ne")
    SET(QNX_CPU_ARCH "armle-v7")
    SET(EB_PLATFORM_ARM_ARCH 7)
endif(CPU_ARM_SOC STREQUAL "Cortex-A8")

if (CPU_ARM_SOC STREQUAL "Cortex-A9")
    MESSAGE(STATUS "CPU_ARM_SOC: Cortex-A9")
    SET(CPU_ARM_ARCH "armv7-a")
    SET(CPU_QNX_PATH "armv7-le")
    SET(CPU_ARM_SPECIFIC "cortex-a9")
    SET(CPU_ARM_FLOAT_ABI "softfp")
    SET(CPU_ARM_FPU "neon")
    SET(QCC_GCC_BACKEND_TYPE "gcc_ntoarmv7le_cpp-ne")
    SET(QNX_CPU_ARCH "armle-v7")
    SET(EB_PLATFORM_ARM_ARCH 7)
endif(CPU_ARM_SOC STREQUAL "Cortex-A9")

if (CPU_ARM_SOC STREQUAL "Cortex-A15")
    MESSAGE(STATUS "CPU_ARM_SOC: Cortex-A15")
    SET(CPU_ARM_ARCH "armv7-a")
    SET(CPU_QNX_PATH "armv7-le")
    SET(CPU_ARM "cortex-a15")
    SET(CPU_ARM_SPECIFIC "cortex-a15")
    SET(CPU_ARM_FLOAT_ABI "softfp")
    SET(CPU_ARM_FPU "neon-vfpv4")
    SET(QCC_GCC_BACKEND_TYPE "gcc_ntoarmv7le_cpp-ne")
    SET(QNX_CPU_ARCH "armle-v7")
    SET(EB_PLATFORM_ARM_ARCH 7)
endif(CPU_ARM_SOC STREQUAL "Cortex-A15")

if (NOT CPU_ARM_SOC STREQUAL ${CPU_ARM_SOC_SAVE})
    unset(GTF_SCDR_OGLES_INCLUDE_DIR CACHE)
    unset(GTF_SCDR_OGLES_LIBRARY CACHE)
    unset(GTF_SCDR_EGL_LIBRARY CACHE)
    unset(OVG_INCLUDE_DIR CACHE)
    unset(OVG_LIBRARY CACHE)
    unset(EGL_LIBRARY CACHE)
    set(CPU_ARM_SOC_SAVE ${CPU_ARM_SOC})
endif()

SET(CMAKE_CXX_COMPILER "${QNX_HOST_CMAKE_PATH}/usr/bin/qcc.exe" CACHE STRING "" FORCE)
SET(CMAKE_C_COMPILER "${QNX_HOST_CMAKE_PATH}/usr/bin/qcc.exe" CACHE STRING "" FORCE)
SET(CMAKE_LINKER "${QNX_HOST_CMAKE_PATH}/usr/bin/qcc.exe" CACHE STRING "" FORCE)
SET(CMAKE_AR "${QNX_HOST_CMAKE_PATH}/usr/bin/ntoarmv7-ar.exe" CACHE STRING "" FORCE)
SET(CMAKE_OBJCOPY "${QNX_HOST_CMAKE_PATH}/usr/bin/ntoarmv7-objcopy.exe" CACHE STRING "" FORCE)
SET(CMAKE_NM "${QNX_HOST_CMAKE_PATH}/usr/bin/ntoarmv7-nm.exe" CACHE STRING "" FORCE)
SET(CMAKE_RANLIB "${QNX_HOST_CMAKE_PATH}/usr/bin/ntoarmv7-ranlib.exe" CACHE STRING "" FORCE)
SET(CMAKE_OBJDUMP "${QNX_HOST_CMAKE_PATH}/usr/bin/ntoarmv7-objdump.exe" CACHE STRING "" FORCE)
SET(CMAKE_STRIP "${QNX_HOST_CMAKE_PATH}/usr/bin/ntoarmv7-strip.exe" CACHE STRING "" FORCE)

if (CPU_ARM STREQUAL "")
    SET(CMAKE_CXX_FLAGS "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -march=${CPU_ARM_ARCH} -mtune=${CPU_ARM_SPECIFIC} -mfloat-abi=${CPU_ARM_FLOAT_ABI} -mfpu=${CPU_ARM_FPU} -fno-stack-protector " CACHE STRING "" FORCE)
    SET(CMAKE_C_FLAGS   "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -march=${CPU_ARM_ARCH} -mtune=${CPU_ARM_SPECIFIC} -mfloat-abi=${CPU_ARM_FLOAT_ABI} -mfpu=${CPU_ARM_FPU} -fno-stack-protector " CACHE STRING "" FORCE)
else()
    SET(CMAKE_CXX_FLAGS "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -mcpu=${CPU_ARM} -mfloat-abi=${CPU_ARM_FLOAT_ABI} -mfpu=${CPU_ARM_FPU} -fno-stack-protector" CACHE STRING "" FORCE)
    SET(CMAKE_C_FLAGS   "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -mcpu=${CPU_ARM} -mfloat-abi=${CPU_ARM_FLOAT_ABI} -mfpu=${CPU_ARM_FPU} -fno-stack-protector" CACHE STRING "" FORCE)
endif()

SET(CMAKE_SHARED_LIBRARY_C_FLAGS "-Wc,-fPIC")
SET(CMAKE_SHARED_LIBRARY_CXX_FLAGS "-Wc,-fPIC")

SET(CMAKE_SHARED_LIBRARY_CREATE_CXX_FLAGS "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -fPIC -shared")
SET(CMAKE_SHARED_LIBRARY_CREATE_C_FLAGS "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -fPIC -shared")

SET(CMAKE_SHARED_LIBRARY_LINK_C_FLAGS "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -Bdynamic")
SET(CMAKE_SHARED_LIBRARY_LINK_CXX_FLAGS "-V${QCC_GCC_BACKEND_VERSION},${QCC_GCC_BACKEND_TYPE} -Bdynamic")

SET(CMAKE_CXX_LINK_EXECUTABLE "<CMAKE_LINKER> <CMAKE_CXX_LINK_FLAGS> <LINK_FLAGS> <OBJECTS> -o <TARGET> <LINK_LIBRARIES>")

SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -lm")
SET(CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -lm")

set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
