////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_EFFECT_RENDER_OBJECT_H_INCLUDED
#define CUSTOM_EFFECT_RENDER_OBJECT_H_INCLUDED

#include <gtf/scdr/ConstantBuffer.h>
#include <gtf/scdr/GraphicsContext.h>
#include <gtf/scdr/PipelineStateObject.h>
#include <gtf/scdr/RenderObject.h>
#include <gtf/scdr/RenderObjectUtils.h>
#include <gtf/scdr/ResourceBindingTable.h>
#include <gtf/scdr/TouchObject.h>
#include <gtf/scdr/VertexBuffer.h>

#include <gtf/widgetset/WidgetModel.h>

#include "CustomEffectStackFrame.h"

class CustomEffectRenderObject : public gtf::scdr::RenderObject, public gtf::scdr::TouchObject
{
public:
    static CustomEffectRenderObject* create(const gtf::widgetset::WidgetModel& widget_, const gtf::scdr::WidgetFeaturesCommon2D& features_, uint32_t imageCount_, uint32_t constantBufferSize_, gtf::scdr::GraphicsContext& gc_, bool isOffscreen_);
    static void destroy(CustomEffectRenderObject* object_, gtf::scdr::GraphicsContext& gc_);

private:
    CustomEffectRenderObject(const CustomEffectRenderObject&);
    CustomEffectRenderObject& operator=(const CustomEffectRenderObject&);

public:
    CustomEffectRenderObject(const gtf::widgetset::WidgetModel& widget_, const gtf::scdr::WidgetFeaturesCommon2D& features_);

public: // RenderObject interface
    virtual gtf::scdr::RenderObjectStackFrame* createStackFrame(gtf::scdr::RenderObjectStackFrame& parent_, gtf::scdr::Allocator& allocator_);
    virtual void destroyStackFrame(gtf::scdr::RenderObjectStackFrame* frame_, gtf::scdr::Allocator& allocator_);

    virtual bool transformCoordinates(gtf::scdr::TransformMode::Value mode_, uint32_t coordCount_, gtf::scdr::TransformCoordinate* coordInOut_) const;

    virtual const gtf::scdr::TouchObject* getTouchObject();

public: // TouchObject interface
    virtual void getBoundingRectangle(gtf::scdr::Rectangle<int16_t>& rectOut_) const;

    virtual bool intersect(const gtf::scdr::ScreenPosition& pos_) const;

public:
    CustomEffectRenderObject*               next;
    gtf::scdr::WidgetFeaturesCommon2DModels features;

    void*                                   buffer;
    gtf::scdr::ImageResourceHandle*         images;

    gtf::scdr::Framebuffer*                 fb;
    gtf::scdr::Framebuffer*                 offscreenFb;
    gtf::scdr::Texture2D**                  textures;
    gtf::scdr::Viewport                     viewport;
    gtf::scdr::Scissor                      scissor;
    gtf::scdr::Color                        coloration;
    gtf::math::float4x4                     objViewProjMat;

    gtf::math::float4x4                     unit2Screen;
    uint32_t                                width;
    uint32_t                                height;

    uint32_t                                fbId;
    uint32_t                                offscreenFbId;
    uint32_t                                drawId;
    uint32_t                                imageCount;
    bool                                    isOffscreen;

};

#endif
