////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_EFFECT_UTILS_H_INCLUDED
#define CUSTOM_EFFECT_UTILS_H_INCLUDED

#include <gtf/scdr/Framebuffer.h>
#include <gtf/scdr/GraphicsExecutionContext.h>

#include <gtf/type/TypeId.h>

#include <gtf/types/TypesC99.h>

#include <gtf/viewfactory/ViewFactory.h>

#include "CustomEffectGraphicObjects.h"
#include "CustomEffectRenderer.h"

struct CustomVertex
{
    float posX;
    float posY;
    float texU;
    float texV;
};

// Helper structure to embrace UserData and RenderData
struct RenderFunctionContext
{
    CustomEffectGraphicObjects* graphicObjects;
    CustomEffectRenderObject*   renderObject;
    CustomEffectRenderer*       renderer;
};

uint32_t Align(uint32_t size_, uint32_t alignment_);

void SetSamplerBinding(gtf::scdr::ResourceBindingDescriptor& desc_, const char* name_, uint32_t slot_);
void SetConstantBufferBinding(gtf::scdr::ResourceBindingDescriptor& desc_, const char* name_, uint32_t offset_);
void SetTexture2DBinding(gtf::scdr::ResourceBindingDescriptor& desc_, const char* name_, uint32_t slot_);
void SetScalarUniformVariable(ScalarUniformVariable& var_, gtf::type::TypeId::typeId_t type_, uint16_t offset_, uint16_t handle_);
void SetImageVariable(ImageVariable& var_, uint16_t slot_, uint16_t handle_);

uint16_t ResolvePropertyHandle(const gtf::viewfactory::ViewFactory& factory_, const char* widgetName_, const char* propertyName_);

bool ClearOffscreenFramebuffer(void* customEffectRenderObject_, gtf::scdr::GraphicsExecutionContext& context_);
bool DeleteRenderFunctionContext(void* memory_, gtf::scdr::GraphicsExecutionContext& context_);

#include "CustomEffectUtils.inl"

#endif
