////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_EFFECT_DIRECTX_SHADER_H_INCLUDED
#define CUSTOM_EFFECT_DIRECTX_SHADER_H_INCLUDED

static const char* VertexShaderD3D11 =
"float4x4 scdrViewProjMat;                                                   "
"                                                                            "
"struct VS_IN                                                                "
"{                                                                           "
"    float2 pos : aPosition0;                                                "
"    float2 tex : aTexCoord0;                                                "
"};                                                                          "
"struct VS_OUT                                                               "
"{                                                                           "
"    float4 pos : SV_POSITION;                                               "
"    float2 tex : TEXCOORD0;                                                 "
"};                                                                          "
"VS_OUT main(VS_IN vsIn)                                                     "
"{                                                                           "
"    VS_OUT vsOut;                                                           "
"    vsOut.pos = mul(scdrViewProjMat, float4(vsIn.pos, 0, 1));               "
"    vsOut.tex = float2(vsIn.tex.x, vsIn.tex.y);                             "
"    return vsOut;                                                           "
"}                                                                           ";

static const char* BlurPixelShaderD3D11 =
"Texture2D<float4> scdrFramebuffer : register(t0);                           "
"SamplerState scdrSampler : register(s0);                                    "
"float2 scdrSize;                                                            "
"int filterSupport;                                                          "
"                                                                            "
"struct PS_IN                                                                "
"{                                                                           "
"    float4 pos : SV_POSITION;                                               "
"    float2 tex : TEXCOORD00;                                                "
"};                                                                          "
"                                                                            "
"float4 main(PS_IN psIn) : SV_TARGET0                                        "
"{                                                                           "
"    if (filterSupport == 0)                                                 "
"    {                                                                       "
"        return scdrFramebuffer.Sample(scdrSampler, psIn.tex);               "
"    }                                                                       "
"                                                                            "
"    float2 invSize = rcp(scdrSize);                                         "
"    float4 sum = 0;                                                         "
"                                                                            "
"    for (int x = -filterSupport; x <= filterSupport; ++x)                   "
"    {                                                                       "
"        for (int y = -filterSupport; y <= filterSupport; ++y)               "
"        {                                                                   "
"            float2 uv = psIn.tex + float2(x, y) * invSize;                  "
"            sum += scdrFramebuffer.Sample(scdrSampler, uv);                 "
"        }                                                                   "
"    }                                                                       "
"                                                                            "
"    return sum / (float)((2 * filterSupport + 1) * (2 * filterSupport + 1));"
"}                                                                           ";

static const char* GaussianBlurPixelShaderD3D11 =
"struct PS_IN                                                                "
"{                                                                           "
"    float4 pos : SV_POSITION;                                               "
"    float2 tex : TEXCOORD0;                                                 "
"};                                                                          "
"                                                                            "
"Texture2D<float4> scdrFramebuffer;                                          "
"SamplerState scdrSampler;                                                   "
"float2 scdrSize;                                                            "
"float sigma;                                                                "
"                                                                            "
"float gauss(float2 xy)                                                      "
"{                                                                           "
"    return exp(-(dot(xy, xy)) / (2.0 * sigma * sigma));                     "
"}                                                                           "
"                                                                            "
"float4 main(PS_IN psIn) : SV_TARGET                                         "
"{                                                                           "
"    if (sigma <= 0.0)                                                       "
"    {                                                                       "
"        return scdrFramebuffer.Sample(scdrSampler, psIn.tex);               "
"    }                                                                       "
"                                                                            "
"    int cnt = (int)ceil(sigma * 3.0);                                       "
"    float2 invSize = rcp(scdrSize);                                         "
"    float4 sum = 0;                                                         "
"    float w = 0;                                                            "
"                                                                            "
"    for (int x = -cnt; x <= cnt; ++x)                                       "
"    {                                                                       "
"        for (int y = -cnt; y <= cnt; ++y)                                   "
"        {                                                                   "
"            float2 fxy = float2(x, y);                                      "
"            float2 uv = psIn.tex + fxy * invSize;                           "
"            float lW = gauss(fxy);                                          "
"            float4 fbV = scdrFramebuffer.Sample(scdrSampler, uv);           "
"                                                                            "
"            sum += lW * fbV;                                                "
"            w += lW;                                                        "
"        }                                                                   "
"    }                                                                       "
"                                                                            "
"    return sum / w;                                                         "
"}                                                                           ";

static const char* SuperSimplePixelShaderD3D11 =
"struct PS_IN                                                                "
"{                                                                           "
"    float4 pos : SV_POSITION0;                                              "
"    float2 tex : TEXCOORD0;                                                 "
"};                                                                          "
"                                                                            "
"float4 scdrColoration;                                                      "
"                                                                            "
"float4 main(PS_IN psIn) : SV_TARGET0                                        "
"{                                                                           "
"    return scdrColoration;                                                  "
"}                                                                           ";

static const char* SepiaPixelShaderD3D11 =
"struct PS_IN                                                                "
"{                                                                           "
"    float4 pos : SV_POSITION0;                                              "
"    float2 tex : TEXCOORD0;                                                 "
"};                                                                          "
"                                                                            "
"float4 scdrColoration;                                                      "
"Texture2D<float4> image;                                                    "
"SamplerState scdrSampler;                                                   "
"                                                                            "
"float4 main(PS_IN psIn) : SV_TARGET0                                        "
"{                                                                           "
"    const float3 SEPIA = float3(1.2, 1.0, 0.8);                             "
"                                                                            "
"    float4 c = image.Sample(scdrSampler, psIn.tex);                         "
"    c *= scdrColoration;                                                    "
"                                                                            "
"    float grey = dot(c.xyz, float3(0.299, 0.587, 0.114));                   "
"    c.xyz = grey * SEPIA;                                                   "
"                                                                            "
"    return c;                                                               "
"}                                                                           ";

static const char* ProgressbarPixelShaderD3D11 =
"struct PS_IN                                                                "
"{                                                                           "
"    float4 pos : SV_POSITION0;                                              "
"    float2 tex : TEXCOORD0;                                                 "
"};                                                                          "
"                                                                            "
"Texture2D<float4> bar;                                                      "
"Texture2D<float4> coloration;                                               "
"SamplerState scdrSampler;                                                   "
"                                                                            "
"float progress;                                                             "
"bool dynamicColoration;                                                     "
"float4 main(PS_IN psIn) : SV_TARGET0                                        "
"{                                                                           "
"    float2 mask = bar.Sample(scdrSampler, psIn.tex).xy;                     "
"    float2 colLookUp = float2(mask.y, 0.5);                                 "
"    if(dynamicColoration)                                                   "
"    {                                                                       "
"       colLookUp.x = progress/100.0;                                        "
"    }                                                                       "
"    float4 color = coloration.Sample(scdrSampler, colLookUp);               "
"    color = color * mask.x * step(mask.y, progress/100.0);                  "
"    return color;                                                           "
"}                                                                           ";

#endif
