////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_EFFECT_GLES20_SHADER_H_INCLUDED
#define CUSTOM_EFFECT_GLES20_SHADER_H_INCLUDED

static const char* VertexShaderGLES20 =
"uniform highp mat4 scdrViewProjMat;                                               "
"                                                                                  "
"attribute highp vec2 aPosition;                                                   "
"attribute highp vec2 aTexCoord;                                                   "
"                                                                                  "
"varying mediump vec2 vTexCoord;                                                   "
"                                                                                  "
"void main()                                                                       "
"{                                                                                 "
"    vTexCoord   = vec2(aTexCoord.x, 1.0 - aTexCoord.y);                           "
"    vec4 pos    = scdrViewProjMat * vec4(aPosition, 0.0, 1.0);                    "
"    gl_Position = pos;                                                            "
"}                                                                                 ";

static const char* FragmentShaderGLES20 =
"uniform sampler2D scdrFramebuffer;                                                "
"                                                                                  "
"varying mediump vec2 vTexCoord;                                                   "
"                                                                                  "
"void main()                                                                       "
"{                                                                                 "
"        gl_FragColor = texture2D(scdrFramebuffer,vTexCoord);                      "
"}                                                                                 ";

static const char* SuperSimpleFragmentShaderGLES20 =
"uniform lowp vec4 scdrColoration;                                                 "
"                                                                                  "
"varying mediump vec2 vTexCoord;                                                   "
"                                                                                  "
"void main()                                                                       "
"{                                                                                 "
"    gl_FragColor = scdrColoration;                                                "
"}                                                                                 ";

static const char* SepiaFragmentShaderGLES20 =
"uniform lowp vec4 scdrColoration;                                                 "
"uniform sampler2D image;                                                          "
"varying mediump vec2 vTexCoord;                                                   "
"                                                                                  "
"void main()                                                                       "
"{                                                                                 "
"    highp vec4 c = texture2D(image, vec2(vTexCoord.x, 1.0 - vTexCoord.y));        "
"    c *= scdrColoration;                                                          "
"    highp float grey = dot(c.xyz, vec3(0.299, 0.587, 0.114));                     "
"    c.xyz = grey * vec3(1.2, 1.0, 0.8);                                           "
"                                                                                  "
"    gl_FragColor = c;                                                             "
"}                                                                                 ";

static const char* ProgressbarFragmentShaderGLES20 =
"uniform sampler2D bar;                                                         "
"uniform sampler2D coloration;                                                  "
"uniform lowp float progress;                                                   "
"uniform bool dynamicColoration;                                                "
"                                                                               "
"varying mediump vec2 vTexCoord;                                                "
"                                                                               "
"void main()                                                                    "
"{                                                                              "
"    highp vec2 mask = texture2D(bar, vec2(vTexCoord.x, 1.0 - vTexCoord.y)).xy; "
"    highp vec2 colLookUp = vec2(mask.y, 0.5);                                  "
"    if(dynamicColoration)                                                      "
"    {                                                                          "
"        colLookUp.x = progress/100.0;                                          "
"    }                                                                          "
"    highp vec4 color = texture2D(coloration, colLookUp);                       "
"    color = color * mask.x * step(mask.y, progress/100.0);                     "
"                                                                               "
"    gl_FragColor = color;                                                      "
"}                                                                              ";

#endif
