////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_EFFECT_GLES3_SHADER_H_INCLUDED
#define CUSTOM_EFFECT_GLES3_SHADER_H_INCLUDED

static const char* VertexShaderGLES3 =
"uniform highp mat4 scdrViewProjMat;                                                    "
"                                                                                       "
"layout (location = 0) in highp vec2 aPosition;                                         "
"layout (location = 1) in highp vec2 aTexCoord;                                         "
"                                                                                       "
"out mediump vec2 vTexCoord;                                                            "
"                                                                                       "
"void main()                                                                            "
"{                                                                                      "
"    vec4 pos    = vec4(aPosition, 0.0, 1.0);                                           "
"    gl_Position = scdrViewProjMat * pos;                                               "
"    vTexCoord   = vec2(aTexCoord.x, 1.0 - aTexCoord.y);                                "
"}                                                                                      ";

static const char* BlurFragmentShaderGLES3 =
"uniform sampler2D  scdrFramebuffer;                                                    "
"uniform highp vec2 scdrSize;                                                           "
"uniform int        filterSupport;                                                      "
"                                                                                       "
"in mediump vec2 vTexCoord;                                                             "
"out highp vec4  fragmentColor;                                                         "
"                                                                                       "
"void main()                                                                            "
"{                                                                                      "
"    if (filterSupport <= 0)                                                            "
"    {                                                                                  "
"        fragmentColor = texture(scdrFramebuffer, vTexCoord);                           "
"        return;                                                                        "
"    }                                                                                  "
"                                                                                       "
"    highp vec2 invSize = 1.0 / scdrSize;                                               "
"    highp vec4 sum = vec4(0.0);                                                        "
"                                                                                       "
"    for (int x = -filterSupport; x <= filterSupport; ++x)                              "
"    {                                                                                  "
"        for (int y = -filterSupport; y <= filterSupport; ++y)                          "
"        {                                                                              "
"            mediump vec2 uv = vTexCoord + vec2(x, y) * invSize;                        "
"            sum += texture(scdrFramebuffer, uv);                                       "
"        }                                                                              "
"    }                                                                                  "
"    fragmentColor = sum / vec4((2 * filterSupport + 1) * (2 * filterSupport + 1));     "
"}                                                                                      ";


static const char* GaussianBlurFragmentShaderGLES3 =
"uniform sampler2D   scdrFramebuffer;                                                   "
"uniform highp vec2  scdrSize;                                                          "
"uniform highp float sigma;                                                             "
"                                                                                       "
"highp float gauss(highp vec2 xy)                                                       "
"{                                                                                      "
"    return exp(-(dot(xy, xy)) / (2.0 * sigma * sigma));                                "
"}                                                                                      "
"                                                                                       "
"in mediump vec2 vTexCoord;                                                             "
"out highp vec4  fragColor;                                                             "
"                                                                                       "
"void main()                                                                            "
"{                                                                                      "
"    if (sigma <= 0.0)                                                                  "
"    {                                                                                  "
"        fragColor = texture(scdrFramebuffer, vTexCoord);                               "
"        return;                                                                        "
"    }                                                                                  "
"                                                                                       "
"    highp int cnt = int(ceil(sigma * 3.0));                                            "
"    highp vec2 invSize = vec2(1.0) / scdrSize;                                         "
"    highp vec4 cSum = vec4(0.0);                                                       "
"    highp float wSum = 0.0;                                                            "
"                                                                                       "
"    for (int x = -cnt; x <= cnt; ++x)                                                  "
"    {                                                                                  "
"        for (int y = -cnt; y <= cnt; ++y)                                              "
"        {                                                                              "
"            highp vec2 fxy = vec2(x, y);                                               "
"            mediump vec2 uv = vTexCoord + fxy * invSize;                               "
"                                                                                       "
"            highp float w = gauss(fxy);                                                "
"            highp vec4 c = texture(scdrFramebuffer, uv);                               "
"                                                                                       "
"            cSum += w * c;                                                             "
"            wSum += w;                                                                 "
"        }                                                                              "
"    }                                                                                  "
"                                                                                       "
"    fragColor = cSum / wSum;                                                           "
"}                                                                                      ";



static const char* SuperSimpleFragmentShaderGLES3 =
"uniform lowp vec4 scdrColoration;                                                      "
"in mediump vec2 vTexCoord;                                                             "
"                                                                                       "
"out highp vec4 fragmentColor;                                                          "
"                                                                                       "
"void main()                                                                            "
"{                                                                                      "
"    fragmentColor = scdrColoration;                                                    "
"}                                                                                      ";

static const char* SepiaFragmentShaderGLES3 =
"uniform lowp vec4 scdrColoration;                                                      "
"uniform sampler2D image;                                                               "
"in mediump vec2 vTexCoord;                                                             "
"out highp vec4 fragmentColor;                                                          "
"                                                                                       "
"void main()                                                                            "
"{                                                                                      "
"    highp vec4 c = texture(image, vec2(vTexCoord.x, 1.0 - vTexCoord.y));               "
"    c *= scdrColoration;                                                               "
"    highp float grey = dot(c.xyz, vec3(0.299, 0.587, 0.114));                          "
"    c.xyz = grey * vec3(1.2, 1.0, 0.8);                                                "
"                                                                                       "
"    fragmentColor = c;                                                                 "
"}                                                                                      ";

static const char* ProgressbarFragmentShaderGLES3 =
"uniform sampler2D bar;                                                                 "
"uniform sampler2D coloration;                                                          "
"uniform lowp float progress;                                                           "
"uniform bool dynamicColoration;                                                        "
"                                                                                       "
"in mediump vec2 vTexCoord;                                                             "
"out highp vec4 fragmentColor;                                                          "
"                                                                                       "
"void main()                                                                            "
"{                                                                                      "
"    highp vec2 mask = texture(bar, vec2(vTexCoord.x, 1.0 - vTexCoord.y)).xy;           "
"    highp vec2 colLookUp = vec2(mask.y, 0.5);                                          "
"    if(dynamicColoration)                                                              "
"    {                                                                                  "
"        colLookUp.x = progress/100.0;                                                  "
"    }                                                                                  "
"    highp vec4 color = texture(coloration, colLookUp);                                 "
"    color = color * mask.x * step(mask.y, progress/100.0);                             "
"                                                                                       "
"    fragmentColor = color;                                                             "
"}                                                                                      ";

#endif
