////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_WIDGET_RIPPLE_EFFECT_DIRECTX_SHADER_H_INCLUDED
#define CUSTOM_WIDGET_RIPPLE_EFFECT_DIRECTX_SHADER_H_INCLUDED

static const char* VertexShaderD3D11 =
"cbuffer RippleParameters : register(b0)                          "
"{                                                                "
"    float4x4 projMat;                                            "
"    float4x4 viewMat;                                            "
"    float time;                                                  "
"    float active;                                                "
"};                                                               "
"struct VS_IN                                                     "
"{                                                                "
"    float2 pos : aPosition0;                                     "
"    float2 tex : aTexCoord0;                                     "
"};                                                               "
"struct VS_OUT                                                    "
"{                                                                "
"    float4 pos : SV_POSITION;                                    "
"    float2 tex : TEXCOORD0;                                      "
"};                                                               "
"VS_OUT main(VS_IN vsIn)                                          "
"{                                                                "
"    VS_OUT vsOut;                                                "
"    float4 view = mul(viewMat, float4(vsIn.pos, 0, 1));          "
"    vsOut.pos = mul(projMat, view);                              "
"    vsOut.tex = float2(vsIn.tex.x, vsIn.tex.y);                  "
"    return vsOut;                                                "
"}                                                                ";

static const char* PixelShaderD3D11 =
"cbuffer RippleParameters : register(b0)                                            "
"{                                                                                  "
"    float4x4 projMat;                                                              "
"    float4x4 viewMat;                                                              "
"    float time;                                                                    "
"    float active;                                                                  "
"}                                                                                  "
"                                                                                   "
"Texture2D<float4> tex : register(t0);                                              "
"SamplerState texSampler : register(s0);                                            "
"                                                                                   "
"struct PS_IN                                                                       "
"{                                                                                  "
"    float4 pos : SV_POSITION;                                                      "
"    float2 tex : TEXCOORD0;                                                        "
"};                                                                                 "
"struct PS_OUT                                                                      "
"{                                                                                  "
"    float4 color : SV_TARGET0;                                                     "
"};                                                                                 "
"PS_OUT main(PS_IN psIn)                                                            "
"{                                                                                  "
"    PS_OUT psOut;                                                                  "
"    float2 p = -1.f + 2.f * psIn.tex;                                              "
"    float l = length(p);                                                           "
"    float2 uv = psIn.tex + active * (p / l) * cos(l * 12 - 4 * time / 100) * 0.03; "
"    float4 texColor = tex.SampleLevel(texSampler, uv, 0);                          "
"    psOut.color = texColor;                                                        "
"    return psOut;                                                                  "
"}                                                                                  ";

#endif // CUSTOM_WIDGET_RIPPLE_EFFECT_DIRECTX_SHADER_H_INCLUDED
